/*
  Copyright (c) 2018 Red Hat, Inc. <http://www.redhat.com>
  This file is part of GlusterFS.

  This file is licensed to you under your choice of the GNU Lesser
  General Public License, version 3 or any later version (LGPLv3 or
  later), or the GNU General Public License, version 2 (GPLv2), in all
  cases as published by the Free Software Foundation.
*/

/* File: utime-autogen-fops-tmpl.h
 * This file contains the utime autogenerated FOPs declarations.
 */

#ifndef _UTIME_AUTOGEN_FOPS_H
#define _UTIME_AUTOGEN_FOPS_H

#include <glusterfs/xlator.h>

#pragma generate

#endif /* _UTIME_AUTOGEN_FOPS_H */
